;;
;; util-shell-commands.el
;;
;; Copyright 2005 Tero Hasu
;;

(setq elruby-script (concat usc-home "/elruby"))

(defun usc-on-first-line ()
  (save-excursion
    (beginning-of-line)
    (= (point) (point-min))))

(defun usc-on-last-line ()
  (save-excursion
    (end-of-line)
    (= (point) (point-max))))

(defun usc-line-start-matches-regexp (regexp)
  (save-excursion
    (beginning-of-line)
    (looking-at regexp)))

(defun usc-on-ruby-comment-line ()
  (usc-line-start-matches-regexp "^[ \t]*#"))

(defun usc-on-lisp-comment-line ()
  (usc-line-start-matches-regexp "^[ \t]*;"))

(defun usc-on-tex-comment-line ()
  (usc-line-start-matches-regexp "^[ \t]*%"))

(defun usc-on-cxx-comment-line ()
  (usc-line-start-matches-regexp "^[ \t]*//"))

(defun usc-on-haskell-comment-line ()
  (usc-line-start-matches-regexp "^[ \t]*--"))

(defun usc-on-ruby-rd-begin-line ()
  (usc-line-start-matches-regexp "^=begin"))
(defun usc-on-ruby-rd-end-line ()
  (usc-line-start-matches-regexp "^=end"))

;; Note the double-espaced asterisks here; the double escapes are
;; required since we are dealing with strings here, but want regexps.
(defun usc-on-cxx-bc-begin-line ()
  (usc-line-start-matches-regexp "^[ \t]*/\\*+[ \t]*$"))
(defun usc-on-cxx-bc-end-line ()
  (usc-line-start-matches-regexp "^[ \t]*\\*+/[ \t]*$"))

(defun usc-on-haskell-bc-begin-line ()
  (usc-line-start-matches-regexp "^[ \t]*\{-+[ \t]*$"))
(defun usc-on-haskell-bc-end-line ()
  (usc-line-start-matches-regexp "^[ \t]*-+\}[ \t]*$"))

(defun usc-on-python-ds-delim-line ()
  (usc-line-start-matches-regexp "^[ \t]*\"\"\"[ \t]*$"))

(defun usc-on-mzscheme-bc-begin-line ()
  (usc-line-start-matches-regexp "^[ \t]*#|[ \t]*$"))
(defun usc-on-mzscheme-bc-end-line ()
  (usc-line-start-matches-regexp "^[ \t]*|#[ \t]*$"))

(defun usc-uniform-region-start-pos (is-match)
  (save-excursion
    (if (funcall is-match)
	(progn
	  (while (and (not (usc-on-first-line))
		      (funcall is-match))
	    (forward-line -1))
	  (if (not (funcall is-match))
	      (forward-line 1))
	  (beginning-of-line)
	  (point)))))

(defun usc-uniform-region-end-pos (is-match)
  (save-excursion
    (if (funcall is-match)
	(progn
	  (while (and (not (usc-on-last-line))
		      (funcall is-match))
	    (forward-line 1))
	  (if (funcall is-match)
	      (end-of-line)
	    (beginning-of-line))
	  (point)))))

(defun usc-delim-region-start-pos (is-match)
  (save-excursion
    (while (and (not (usc-on-first-line))
		(not (funcall is-match)))
      (forward-line -1))
    (if (funcall is-match)
	(progn
	  (forward-line 1)
	  (beginning-of-line)
	  (point)))))

(defun usc-delim-region-end-pos (is-match)
  (save-excursion
    (while (and (not (usc-on-last-line))
		(not (funcall is-match)))
      (forward-line 1))
    (if (funcall is-match)
	(progn
	  (beginning-of-line)
	  (point)))))

(defun usc-filter-region (bpos epos cmd)
  (let ((oldpoint (point)))
    (shell-command-on-region bpos epos
			     (format "ruby -I %s %s %s" usc-home elruby-script cmd) nil t)
    (if (and (>= oldpoint (point-min))
	     (<= oldpoint (point-max)))
	(goto-char oldpoint))))

(defun usc-filter-uniform-region (is-match cmd)
  (let ((bpos (usc-uniform-region-start-pos is-match))
	(epos (usc-uniform-region-end-pos is-match)))
    (usc-filter-region bpos epos cmd)))

(defun usc-filter-delim-region (is-beg is-end cmd)
  (let ((bpos (usc-delim-region-start-pos is-beg))
	(epos (usc-delim-region-end-pos is-end)))
    (usc-filter-region bpos epos cmd)))

(defun usc-filter-ruby-comment-region (cmd)
  (usc-filter-uniform-region 'usc-on-ruby-comment-line cmd))

(defun usc-filter-ruby-rd-region (cmd)
  (usc-filter-delim-region 'usc-on-ruby-rd-begin-line
			   'usc-on-ruby-rd-end-line
			   cmd))

(defun usc-filter-cxx-bc-region (cmd)
  (usc-filter-delim-region 'usc-on-cxx-bc-begin-line
			   'usc-on-cxx-bc-end-line
			   cmd))

(defun usc-filter-haskell-bc-region (cmd)
  (usc-filter-delim-region 'usc-on-haskell-bc-begin-line
			   'usc-on-haskell-bc-end-line
			   cmd))

(defun usc-filter-mzscheme-bc-region (cmd)
  (usc-filter-delim-region 'usc-on-mzscheme-bc-begin-line
			   'usc-on-mzscheme-bc-end-line
			   cmd))

(defun usc-filter-python-ds-region (cmd)
  (usc-filter-delim-region 'usc-on-python-ds-delim-line
			   'usc-on-python-ds-delim-line
			   cmd))

(defun usc-filter-lisp-comment-region (cmd)
  (usc-filter-uniform-region 'usc-on-lisp-comment-line cmd))

(defun usc-filter-haskell-comment-region (cmd)
  (usc-filter-uniform-region 'usc-on-haskell-comment-line cmd))

(defun usc-filter-tex-comment-region (cmd)
  (usc-filter-uniform-region 'usc-on-tex-comment-line cmd))

(defun usc-filter-cxx-comment-region (cmd)
  (usc-filter-uniform-region 'usc-on-cxx-comment-line cmd))

(provide 'util-shell-commands)
